@@ -11,18 +11,21 @@ pub fn get_codepoint(buffer: &[u8], i: usize) -> Result<Option<(u32, u8)>, ()> {
 	if let Some(a) = get_byte(buffer, i) {
 		let r = if a & 0x80 == 0x00 {
 			(a, 1)
-		} else if a & 0xE0 == 0xC0 {
+		} else if a & 0xe0 == 0xc0 {
 			let b = expect_byte(buffer, i + 1)?;
-			((a & 0x1F) << 6 | b, 2)
-		} else if a & 0xF0 == 0xE0 {
+			((a & 0x1f) << 6 | (b & 0x3f), 2)
+		} else if a & 0xf0 == 0xe0 {
 			let b = expect_byte(buffer, i + 1)?;
 			let c = expect_byte(buffer, i + 2)?;
-			((a & 0x0F) << 12 | b << 6 | c, 2)
-		} else if a & 0xF8 == 0xF0 {
+			((a & 0x0f) << 12 | (b & 0x3f) << 6 | (c & 0x3f), 3)
+		} else if a & 0xf8 == 0xf0 {
 			let b = expect_byte(buffer, i + 1)?;
 			let c = expect_byte(buffer, i + 2)?;
 			let d = expect_byte(buffer, i + 3)?;
-			((a & 0x07) << 18 | b << 12 | c << 6 | d, 3)
+			(
+				(a & 0x07) << 18 | (b & 0x3f) << 12 | (c & 0x3f) << 6 | (d & 0x3f),
+				4,
+			)
 		} else {
 			return Err(());
 		};
@@ -43,3 +46,24 @@ pub fn get_char(buffer: &[u8], i: usize) -> Result<Option<(char, u8)>, ()> {
 		Err(()) => Err(()),
 	}
 }
+
+#[cfg(test)]
+pub mod tests {
+	use super::*;
+
+	#[test]
+	fn decode() {
+		let string = "伝言/أكرم";
+		let bytes = string.as_bytes();
+
+		let mut index = 0;
+		let mut decoded = String::new();
+		while index < bytes.len() {
+			let (c, i) = get_char(bytes, index).unwrap().unwrap();
+			decoded.push(c);
+			index += i as usize;
+		}
+
+		assert_eq!(decoded, string)
+	}
+}
